@@ -4,14 +4,13 @@ mod after;
 mod and_then;
 mod around;
 mod before;
+mod catch_error;
 #[allow(clippy::module_inception)]
 mod endpoint;
-#[cfg(feature = "staticfiles")]
 mod files;
-mod map_err;
-mod map_ok;
+mod inspect_err;
+mod map;
 mod map_to_response;
-mod map_to_result;
 #[cfg(feature = "prometheus")]
 mod prometheus_exporter;
 #[cfg(feature = "tower-compat")]
@@ -21,13 +20,12 @@ pub use after::After;
 pub use and_then::AndThen;
 pub use around::Around;
 pub use before::Before;
+pub use catch_error::CatchError;
 pub use endpoint::{make, make_sync, BoxEndpoint, Endpoint, EndpointExt, IntoEndpoint};
-#[cfg(feature = "staticfiles")]
 pub use files::Files;
-pub use map_err::MapErr;
-pub use map_ok::MapOk;
+pub use inspect_err::InspectError;
+pub use map::Map;
 pub use map_to_response::MapToResponse;
-pub use map_to_result::MapToResult;
 #[cfg(feature = "prometheus")]
 pub use prometheus_exporter::PrometheusExporter;
 #[cfg(feature = "tower-compat")]
