@@ -6,7 +6,7 @@ use libopentelemetry::{
     Key,
 };
 
-use crate::{Endpoint, IntoResponse, Middleware, Request, Response};
+use crate::{Endpoint, IntoResponse, Middleware, Request, Response, Result};
 
 const METHOD_KEY: Key = Key::from_static_str("request_method");
 const PATH_KEY: Key = Key::from_static_str("request_path");
@@ -76,13 +76,13 @@ pub struct OpenTelemetryMetricsEndpoint<E> {
 impl<E: Endpoint> Endpoint for OpenTelemetryMetricsEndpoint<E> {
     type Output = Response;
 
-    async fn call(&self, req: Request) -> Self::Output {
+    async fn call(&self, req: Request) -> Result<Self::Output> {
         let mut labels = Vec::with_capacity(3);
         labels.push(METHOD_KEY.string(req.method().to_string()));
         labels.push(PATH_KEY.string(req.uri().path().to_string()));
 
         let s = Instant::now();
-        let resp = self.inner.call(req).await.into_response();
+        let resp = self.inner.call(req).await?.into_response();
         let elapsed = s.elapsed();
 
         labels.push(STATUS_KEY.i64(resp.status().as_u16() as i64));
@@ -94,6 +94,6 @@ impl<E: Endpoint> Endpoint for OpenTelemetryMetricsEndpoint<E> {
         self.duration
             .record(elapsed.as_secs_f64() / 1000.0, &labels);
 
-        resp
+        Ok(resp)
     }
 }
