@@ -176,8 +176,10 @@ async fn serve_connection(
             let local_addr = local_addr.clone();
             let remote_addr = remote_addr.clone();
             async move {
-                let resp = ep.call((req, local_addr, remote_addr).into()).await.into();
-                Ok::<_, Infallible>(resp)
+                match ep.call((req, local_addr, remote_addr).into()).await {
+                    Ok(resp) => Ok::<http::Response<_>, Infallible>(resp.into()),
+                    Err(err) => Ok(err.as_response().into()),
+                }
             }
         }
     });
